<?php
/**
 * @copyright   Copyright (C) 2005 - 2012 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * Helper for mod_menuwithitemselection
 *
 * @since       1.5
 */
class modMenuWithItemSelectionHelper
{
	/**
	 * Get a list of the menu items.
	 *
	 * @param  JRegistry   $params  The module options.
	 *
	 * @return  array
	 *
	 * @since	1.5
	 */
	public static function getList(&$params)
	{
		$app = JFactory::getApplication();
		$menu = $app->getMenu();

		// Get active menu item
		$active = self::getActive($params);
		$user = JFactory::getUser();
		$levels = $user->getAuthorisedViewLevels();
		asort($levels);
		$key = 'menu_items' . $params . implode(',', $levels) . '.' . $active->id;
		$cache = JFactory::getCache('mod_menuwithitemselection', '');
		if (!($items = $cache->get($key)))
		{
			$ignoreBasicOptions = $params->get('ignoreBasicOptions');
			$itemsToShow = $params->get('itemsToShow', array());

			$path    = $active->tree;
			$start   = $ignoreBasicOptions ? 1 : (int) $params->get('startLevel');
			$end     = (int) $params->get('endLevel');
			$showAll = $params->get('showAllChildren');
			$items   = $ignoreBasicOptions ? self::getAllItemsOrderedByMenuType() : $menu->getItems('menutype',$params->get('menutype'));

			$lastitem = 0;

			if ($items)
			{
				foreach ($items as $i => $item)
				{
					if (($ignoreBasicOptions && array_search($item->id, $itemsToShow) === false) || (
					    !$ignoreBasicOptions && (
						($start && $start > $item->level)
						|| ($end && $item->level > $end)
						|| (!$showAll && $item->level > 1 && !in_array($item->parent_id, $path))
						|| ($start > 1 && !in_array($item->tree[$start - 2], $path)))))
					{
						unset($items[$i]);
						continue;
					}

					$item->deeper     = false;
					$item->shallower  = false;
					$item->level_diff_to_shallower = 0;
					$item->level_diff_from_shallower = 0;

					if (isset($items[$lastitem]))
					{
						$items[$lastitem]->deeper     = ($item->level > $items[$lastitem]->level);
						$items[$lastitem]->shallower  = ($item->level < $items[$lastitem]->level);
						if ($items[$lastitem]->shallower)
						{
							$items[$lastitem]->level_diff_to_shallower = ($items[$lastitem]->level - $item->level);
						}
						else if ($items[$lastitem]->deeper)
						{
							$item->level_diff_from_shallower = ($item->level - $items[$lastitem]->level);
						}
					}

					$item->parent = (boolean) $menu->getItems('parent_id', (int) $item->id, true);

					$lastitem     = $i;
					$item->active = false;
					$item->flink  = $item->link;

					// Reverted back for CMS version 2.5.6
					switch ($item->type)
					{
						case 'separator':
						case 'heading':
							// No further action needed.
							continue;

						case 'url':
							if ((strpos($item->link, 'index.php?') === 0) && (strpos($item->link, 'Itemid=') === false))
							{
								// If this is an internal Joomla link, ensure the Itemid is set.
								$item->flink = $item->link . '&Itemid=' . $item->id;
							}
							break;

						case 'alias':
							// If this is an alias use the item id stored in the parameters to make the link.
							$item->flink = 'index.php?Itemid=' . $item->params->get('aliasoptions');
							break;

						default:
							$router = JSite::getRouter();
							if ($router->getMode() == JROUTER_MODE_SEF)
							{
								$item->flink = 'index.php?Itemid=' . $item->id;
							}
							else
							{
								$item->flink .= '&Itemid=' . $item->id;
							}
							break;
					}

					if (strcasecmp(substr($item->flink, 0, 4), 'http') && (strpos($item->flink, 'index.php?') !== false))
					{
						$item->flink = JRoute::_($item->flink, true, $item->params->get('secure'));
					}
					else
					{
						$item->flink = JRoute::_($item->flink);
					}

					// We prevent the double encoding because for some reason the $item is shared for menu modules and we get double encoding
					// when the cause of that is found the argument should be removed
					$item->title        = htmlspecialchars($item->title, ENT_COMPAT, 'UTF-8', false);
					$item->anchor_css   = htmlspecialchars($item->params->get('menu-anchor_css', ''), ENT_COMPAT, 'UTF-8', false);
					$item->anchor_title = htmlspecialchars($item->params->get('menu-anchor_title', ''), ENT_COMPAT, 'UTF-8', false);
					$item->menu_image   = $item->params->get('menu_image', '') ? htmlspecialchars($item->params->get('menu_image', ''), ENT_COMPAT, 'UTF-8', false) : '';
				}

				$firstItem = reset($items);
				if ($firstItem)
				{
					//The first item is a special case. As it has no previous
					//item, the level difference must be calculated against the
					//base of the list (which is 1 level below the start level).
					$firstItem->level_diff_from_shallower = $firstItem->level - (($start?$start:1) - 1);
				}

				if (isset($items[$lastitem]))
				{
					$items[$lastitem]->deeper     = (($start?$start:1) > $items[$lastitem]->level);
					$items[$lastitem]->shallower  = (($start?$start:1) < $items[$lastitem]->level);
					$items[$lastitem]->level_diff_to_shallower = ($items[$lastitem]->level - ($start?$start:1));
				}
			}

			$cache->store($items, $key);
		}
		return $items;
	}

	/**
	 * Get base menu item.
	 *
	 * @param   JRegistry  $params  The module options.
	 *
	 * @return   object
	 *
	 * @since	3.0.2
	 */
	public static function getBase(&$params)
	{

		// Get base menu item from parameters
		if ($params->get('base'))
		{
			$base = JFactory::getApplication()->getMenu()->getItem($params->get('base'));
		}
		else
		{
			$base = false;
		}

		// Use active menu item if no base found
		if (!$base)
		{
			$base = self::getActive($params);
		}

		return $base;
	}

	/**
	 * Get active menu item.
	 *
	 * @param   JRegistry  $params  The module options.
	 *
	 * @return  object
	 *
	 * @since	3.0.2
	 */
	public static function getActive(&$params)
	{
		$menu = JFactory::getApplication()->getMenu();

		return $menu->getActive() ? $menu->getActive() : $menu->getDefault();
	}

	private static function getAllItemsOrderedByMenuType()
	{
		$db = JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->select('menutype');
		$query->from('#__menu_types');
		$query->order('title, menutype');
		$db->setQuery($query);

		try
		{
			$menuTypes = $db->loadColumn();    
		}
		catch (JDatabaseException $e)
		{
			return array();
		}

		$app = JFactory::getApplication();
		$menu = $app->getMenu();

		$items = array();
		foreach ($menuTypes as $menutype)
		{
			$items = array_merge($items, $menu->getItems('menutype', $menutype));
		}

		return $items;
	}
}
