<?php
/**
 * @copyright	Copyright (C) 2012 Daniel Calviño Sánchez.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

require_once JPATH_BASE.'/tests/unit/core/case/database.php';
require_once 'PHPUnit/Extensions/Database/DataSet/CsvDataSet.php';

define('JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION', realpath(JPATH_BASE) . '/tests/unit/suites/modules/mod_menuwithitemselection');

class MenuWithItemSelectionTest extends TestCaseDatabase {

    private $menuItems;

    public static function setUpBeforeClass() {
        parent::setUpBeforeClass();

        //The default database schema loaded by the parent class
        //(JPATH_TESTS . '/schema/ddl.sql') does not create the table
        //'jos_template_styles'. The SQL file used here creates that missing
        //table.
        $pdo = self::$driver->getConnection();
        $pdo->exec(file_get_contents(JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION . '/ddl-missing.sql'));
    }

    protected function setUp() {
        parent::setUp();

        //Needed for JURI, which is needed by JRouter
        $_SERVER['HTTP_HOST'] = 'localhost';

        //Set a JSession stub, as creating a real JSession fails (see the
        //comments on JSessionStub class)
        JFactory::$session = new JSessionStub();

        //Instantiate the JApplication before the tests
        JFactory::getApplication('site', array('session' => false));

        //Needed by mod_menuwithitemselection.php
        jimport('joomla.application.module.helper');
        //Needed by JRouter, which is used by modMenuHelper
        jimport('joomla.environment.uri');

        //Reload the menu items, as the items in memory could have been modified
        //and the active item could have been set by the previous test.
        jimport('joomla.application.menu');
        $reflectionProperty = new ReflectionProperty('JMenu', 'instances');
        $reflectionProperty->setAccessible(true);
        $reflectionProperty->setValue(array());

        $menu = JFactory::getApplication()->getMenu();
        $this->menuItems = $menu->getItems(array(), array());
    }

    protected function getDataSet() {
        $prefix = JFactory::$database->getPrefix();

        $dataSet = new PHPUnit_Extensions_Database_DataSet_CsvDataSet(',', "'", '\\');
        $dataSet->addTable($prefix.'extensions', JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION . '/extensions.csv');
        $dataSet->addTable($prefix.'menu', JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION . '/menu.csv');
        $dataSet->addTable($prefix.'menu_types', JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION . '/menu_types.csv');
        $dataSet->addTable($prefix.'template_styles', JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION . '/template_styles.csv');
        $dataSet->addTable($prefix.'usergroups', JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION . '/usergroups.csv');
        $dataSet->addTable($prefix.'viewlevels', JPATH_TESTS_UNIT_MOD_MENUWITHITEMSELECTION . '/viewlevels.csv');
        return $dataSet;
    }

    public function testBasicOptionsFirstLevelDoNotShowAllChildren() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 0);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' && $item->level == 1) {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsFirstLevelDoNotShowAllChildrenFirstLevelActive() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 0);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        $activeItem = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        JFactory::getApplication()->getMenu()->setActive($activeItem->id);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' &&
                ($item->level == 1 ||
                ($item->level == 2 && $item->parent_id == $activeItem->id))) {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsFirstLevelDoNotShowAllChildrenSecondLevelActive() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 0);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        $activeItem = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions'));
        JFactory::getApplication()->getMenu()->setActive($activeItem->id);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' &&
                ($item->level == 1 ||
                ($item->level == 2 && $item->parent_id == $activeItem->parent_id) ||
                ($item->level == 3 && $item->parent_id == $activeItem->id))) {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsSecondLevelDoNotShowAllChildren() {
        $params = new JRegistry;
        $params->set('startLevel', 2);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 0);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        $renderedModule = $this->renderModule($params);

        //There is no active menu item, so the module uses the default menu
        //item, which is the Home menu item in the Main Menu. As the start level
        //is deeper than 1 and there are no menu items in the About Joomla menu
        //that are children of the Home menu item, nothing is shown in the
        //module.
        $this->assertEquals('', $renderedModule, 'No HTML rendered');
    }

    public function testBasicOptionsSecondLevelDoNotShowAllChildrenFirstLevelActive() {
        $params = new JRegistry;
        $params->set('startLevel', 2);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 0);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        $activeItem = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        JFactory::getApplication()->getMenu()->setActive($activeItem->id);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' && $item->level == 2 && $item->parent_id == $activeItem->id) {
                $items[] = $item;
            }
        }

        //Set parent_id of level 2 items to null, as they appear as the root
        //items in the HTML list and thus have no parent
        foreach ($items as $item) {
            if ($item->level == 2) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsSecondLevelDoNotShowAllChildrenSecondLevelActive() {
        $params = new JRegistry;
        $params->set('startLevel', 2);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 0);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        $activeItem = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions'));
        JFactory::getApplication()->getMenu()->setActive($activeItem->id);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' &&
                (($item->level == 2 && $item->parent_id == $activeItem->parent_id) ||
                ($item->level == 3 && $item->parent_id == $activeItem->id))) {
                $items[] = $item;
            }
        }

        //Set parent_id of level 2 items to null, as they appear as the root
        //items in the HTML list and thus have no parent
        foreach ($items as $item) {
            if ($item->level == 2) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsShowAllChildren() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 1);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla') {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsShowAllChildrenMissingFirstLevelItem() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 1);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        //Change the access of an item with children so it is not shown to guest
        //users, but its children items are.
        $itemToHide = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        $itemToHide->access = 2;

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' && $item != $itemToHide) {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        //Set parent_id of hidden item children to the previous item at its same
        //level
        $previousSiblingOfHiddenItem = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            } else if ($item->parent_id == $itemToHide->id) {
                $item->parent_id = $previousSiblingOfHiddenItem->id;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsShowAllChildrenMissingFirstLevelFirstItems() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 1);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        //Change the access of the first level first items, so the first item to
        //appear in the list is a second level item (the last item is a first
        //level item, though).
        $gettingStartedItem = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $gettingStartedItem->access = 2;
        $usingJoomlaItem = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        $usingJoomlaItem->access = 2;

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' &&
                $item != $gettingStartedItem && $item != $usingJoomlaItem) {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        //Set parent_id of hidden item children to null, as the parent has no
        //previous sibling (it is the first item at its level)
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            } else if ($item->parent_id == $usingJoomlaItem->id) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsShowAllChildrenMissingFirstLevelLastItems() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 1);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        //Change the access of the first level last items, so the last item to
        //appear in the list is a second level item (the first item is a first
        //level item, though).
        $theJoomlaProjectItem = $this->getMenuItem('aboutjoomla', array('The Joomla! Project'));
        $theJoomlaProjectItem->access = 2;
        $theJoomlaCommunityItem = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));
        $theJoomlaCommunityItem->access = 2;

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' &&
                $item != $theJoomlaProjectItem && $item != $theJoomlaCommunityItem) {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testBasicOptionsShowAllChildrenMissingSecondLevelItem() {
        $params = new JRegistry;
        $params->set('startLevel', 1);
        $params->set('endLevel', 0);
        $params->set('showAllChildren', 1);
        $params->set('menutype', 'aboutjoomla');
        $params->set('ignoreBasicOptions', 0);

        //Change the access of an item with children so it is not shown to guest
        //users, but its children items are.
        $itemToHide = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions'));
        $itemToHide->access = 2;

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Get items that should appear in the list
        $items = array();
        foreach ($this->menuItems as $item) {
            if ($item->menutype == 'aboutjoomla' && $item != $itemToHide) {
                $items[] = $item;
            }
        }

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        //Set parent_id of hidden item children to null, as the parent has no
        //previous sibling (it is the first item at its level)
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            } else if ($item->parent_id == $itemToHide->id) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSingleMenuSingleRootItem() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of item to null, as it has no parent in the HTML list
        $items[0]->parent_id = null;

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSingleMenuSeveralRootItems() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        $items[] = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            $item->parent_id = null;
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSingleMenuSingleGrandChildItem() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of item to null, as it has no parent in the HTML list
        $items[0]->parent_id = null;

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSingleMenuSingleHierarchy() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components', 'Content Component'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root item to null, as it has no parent in the HTML
        //list
        $items[0]->parent_id = null;

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSingleMenuSeveralHierarchies() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components', 'Content Component'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Beez 2'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Beez5'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Atomic'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Languages'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Parameters'));
        $items[] = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSingleMenuSingleHierarchyWithMissingNodes() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        //Don't add Using Joomla! - Using Extensions node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components', 'Content Component'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root item to null, as it has no parent in the HTML
        //list
        //Set parent_id of hidden item children to null, as the parent has no
        //previous sibling (it is the first item at its level)
        $items[0]->parent_id = null;
        $items[1]->parent_id = null;

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSingleMenuSeveralHierarchiesWithMissingNodes() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        //Don't add Using Joomla! - Using Extensions node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components', 'Content Component'));
        //Don't add Using Joomla! - Using Extensions - Templates node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Beez 2'));
        //Don't add Using Joomla! - Using Extensions - Templates - Beez 5 node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Atomic'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Languages'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Parameters'));
        $items[] = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        //Set parent_id of hidden item children to the previous item at its same
        //level, or null if the parent has no previous siblings
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }
        $items[2]->parent_id = null;
        $items[4]->parent_id = $items[2]->id;
        $items[5]->parent_id = $items[2]->id;
        $items[6]->parent_id = null;

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSeveralMenusSingleRootItem() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        $items[] = $this->getMenuItem('fruitshop', array('Directions'));
        $items[] = $this->getMenuItem('mainmenu', array('Home'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSeveralMenusSeveralRootItems() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $items[] = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));
        $items[] = $this->getMenuItem('fruitshop', array('Welcome'));
        $items[] = $this->getMenuItem('fruitshop', array('Directions'));
        $items[] = $this->getMenuItem('fruitshop', array('Recipes'));
        $items[] = $this->getMenuItem('mainmenu', array('Home'));
        $items[] = $this->getMenuItem('mainmenu', array('Sample Sites'));
        $items[] = $this->getMenuItem('parks', array('Park Blog'));
        $items[] = $this->getMenuItem('parks', array('Image Gallery'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionSeveralMenusSeveralRootItemsCheckingOrder() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $items[] = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));
        $items[] = $this->getMenuItem('parks', array('Park Blog')); //'parks' title is 'Australian Parks'
        $items[] = $this->getMenuItem('parks', array('Image Gallery')); //'parks' title is 'Australian Parks'
        $items[] = $this->getMenuItem('fruitshop', array('Welcome'));
        $items[] = $this->getMenuItem('fruitshop', array('Directions'));
        $items[] = $this->getMenuItem('fruitshop', array('Recipes'));
        $items[] = $this->getMenuItem('mainmenu', array('Home'));
        $items[] = $this->getMenuItem('mainmenu', array('Sample Sites'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //If not ordered by menu name, default menu items order would have been:
        //mainmenu - Home
        //aboutjoomla - Getting Started
        //aboutjoomla - The Joomla! Community
        //parks - Park Blog
        //parks - Image Gallery
        //fruitshop - Welcome
        //fruitshop - Directions
        //mainmenu - Sample Sites
        //fruitshop - Recipes
        foreach ($domDocument->documentElement->childNodes as $itemElement) {
            $linkElement = $itemElement->childNodes->item(0);
            $this->assertEquals('a', $linkElement->tagName, 'Tag name of link');
            $this->assertEquals(1, $linkElement->childNodes->length, 'Number of children of link');
            $this->assertTrue($linkElement->childNodes->item(0) instanceof DOMText, 'Child of link is text');
            $title = $linkElement->childNodes->item(0)->wholeText;

            $menuItem = array_shift($items);
            $this->assertEquals($menuItem->title, $title, 'Title of link');
        }
    }

    public function testItemSelectionSeveralMenusSeveralHierarchiesWithMissingNodes() {
        $params = new JRegistry;
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        //Don't add Using Joomla! - Using Extensions node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components', 'Content Component'));
        //Don't add Using Joomla! - Using Extensions - Templates node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Beez 2'));
        //Don't add Using Joomla! - Using Extensions - Templates - Beez 5 node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Atomic'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Languages'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Parameters'));
        $items[] = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));
        $items[] = $this->getMenuItem('fruitshop', array('Directions'));
        $items[] = $this->getMenuItem('mainmenu', array('Home'));
        $items[] = $this->getMenuItem('mainmenu', array('Site Map'));
        $items[] = $this->getMenuItem('mainmenu', array('Site Map', 'Weblinks'));
        //Don't add Sample Sites node
        $items[] = $this->getMenuItem('mainmenu', array('Sample Sites', 'Parks'));
        $items[] = $this->getMenuItem('mainmenu', array('Sample Sites', 'Shop'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        //Set parent_id of hidden item children to the previous item at its same
        //level, or null if the parent has no previous siblings
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }
        $items[2]->parent_id = null;
        $items[4]->parent_id = $items[2]->id;
        $items[5]->parent_id = $items[2]->id;
        $items[6]->parent_id = null;
        $items[13]->parent_id = $items[11]->id;
        $items[14]->parent_id = $items[11]->id;

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    public function testItemSelectionWithBasicOptionsSetSeveralMenusSeveralHierarchiesWithMissingNodes() {
        $params = new JRegistry;
        $params->set('startLevel', 2);
        $params->set('endLevel', 3);
        $params->set('showAllChildren', 0);
        $params->set('menutype', 'usermenu');
        $params->set('ignoreBasicOptions', 1);

        //Get items that should appear in the list
        $items = array();
        $items[] = $this->getMenuItem('aboutjoomla', array('Getting Started'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!'));
        //Don't add Using Joomla! - Using Extensions node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Components', 'Content Component'));
        //Don't add Using Joomla! - Using Extensions - Templates node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Beez 2'));
        //Don't add Using Joomla! - Using Extensions - Templates - Beez 5 node
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Templates', 'Atomic'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Using Extensions', 'Languages'));
        $items[] = $this->getMenuItem('aboutjoomla', array('Using Joomla!', 'Parameters'));
        $items[] = $this->getMenuItem('aboutjoomla', array('The Joomla! Community'));
        $items[] = $this->getMenuItem('fruitshop', array('Directions'));
        $items[] = $this->getMenuItem('mainmenu', array('Home'));
        $items[] = $this->getMenuItem('mainmenu', array('Site Map'));
        $items[] = $this->getMenuItem('mainmenu', array('Site Map', 'Weblinks'));
        //Don't add Sample Sites node
        $items[] = $this->getMenuItem('mainmenu', array('Sample Sites', 'Parks'));
        $items[] = $this->getMenuItem('mainmenu', array('Sample Sites', 'Shop'));

        $itemIds = array();
        foreach ($items as $item) {
            $itemIds[] = $item->id;
        }
        $params->set('itemsToShow', $itemIds);

        $renderedModule = $this->renderModule($params);

        $this->assertValidHtml($renderedModule);

        $domDocument = $this->getDomDocument($renderedModule);

        //Set parent_id of root items to null, as they have no parent in the
        //HTML list
        //Set parent_id of hidden item children to the previous item at its same
        //level, or null if the parent has no previous siblings
        foreach ($items as $item) {
            if ($item->parent_id == 1) {
                $item->parent_id = null;
            }
        }
        $items[2]->parent_id = null;
        $items[4]->parent_id = $items[2]->id;
        $items[5]->parent_id = $items[2]->id;
        $items[6]->parent_id = null;
        $items[13]->parent_id = $items[11]->id;
        $items[14]->parent_id = $items[11]->id;

        $this->assertHtmlList($domDocument->documentElement, $items);
        $this->assertEquals(0, count($items), 'Items not found in HTML');
    }

    private function getMenuItem($menu, $path, $parentId = 1) {
        foreach ($this->menuItems as $item) {
            if ($item->menutype == $menu && $item->title == $path[0] && $item->parent_id == $parentId) {
                if (count($path) == 1) {
                    return $item;
                } else {
                    $childPath = $path;
                    array_shift($childPath);
                    return $this->getMenuItem($menu, $childPath, $item->id);
                }
            }
        }

        return null;
    }

    private function renderModule($params) {
        ob_start();
        require JPATH_BASE.'/modules/mod_menuwithitemselection/mod_menuwithitemselection.php';
        $result = ob_get_contents();
        ob_end_clean();

        return $result;
    }

    private function assertValidHtml($renderedModule) {
        $htmlWrap = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"'
            . ' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
            . '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-gb" lang="en-gb">'
            . '<head><title>Hello world</title></head>'
            . '<body>'
            . $renderedModule
            . '</body></html>';

        $tidy = tidy_parse_string($htmlWrap);
        $this->assertEquals(0, tidy_error_count($tidy), "Tidy errors:\n".$tidy->errorBuffer);
        $this->assertEquals(0, tidy_warning_count($tidy), "Tidy warnings:\n".$tidy->errorBuffer);
    }

    private function getDomDocument($renderedModule) {
        $domDocument = new DOMDocument;
        $domDocument->preserveWhiteSpace = FALSE;
        $domDocument->loadXML($renderedModule);
        $domDocument->formatOutput = TRUE;

        return $domDocument;
    }

    private function assertHtmlList($element, &$items, $parentListItemId = null) {
        $this->assertEquals('ul', $element->tagName, 'List tag');

        foreach ($element->childNodes as $itemElement) {
            $this->assertHtmlListItem($itemElement, $items, $parentListItemId);
        }
    }

    private function assertHtmlListItem($element, &$items, $parentListItemId = null) {
        $this->assertEquals('li', $element->tagName, 'List item tag');

        if ($element->childNodes->length == 0 ||
            ($element->childNodes->length == 1 && $element->childNodes->item(0)->tagName != 'a' &&
                                                 $element->childNodes->item(0)->tagName != 'ul') ||
            ($element->childNodes->length == 2 && ($element->childNodes->item(0)->tagName != 'a' ||
                                                  $element->childNodes->item(1)->tagName != 'ul')) ||
            $element->childNodes->length > 2) {
            $this->assertTrue(false, 'Valid children');
        }

        if ($element->childNodes->length == 1 && $element->childNodes->item(0)->tagName == 'ul') {
            $this->assertHtmlList($element->childNodes->item(0), $items, null);
            return;
        }

        $this->assertEquals(1, $element->attributes->length, 'Number of attributes of list item');
        $this->assertEquals('class', $element->attributes->item(0)->nodeName, 'Attribute "class" of list item is present');
        $this->assertEquals(1, preg_match('#item-(\d+)#', $element->attributes->item(0)->nodeValue, $matches), 'Attribute "class" of list item matches "item-\d+"');
        $id = $matches[1];

        $linkElement = $element->childNodes->item(0);
        $this->assertEquals(1, $linkElement->childNodes->length, 'Number of children of link');
        $this->assertTrue($linkElement->childNodes->item(0) instanceof DOMText, 'Child of link is text');
        $title = $linkElement->childNodes->item(0)->wholeText;

        $index = $this->getMenuItemIndex($id, $title, $items);
        $this->assertNotNull($index, 'Menu item ('.$id.', '.$title.') not found in items list');

        $currentItem = $items[$index];
        $this->assertEquals($parentListItemId, $currentItem->parent_id, 'Parent id of current item');

        if ($element->childNodes->length == 2) {
            $this->assertHtmlList($element->childNodes->item(1), $items, $id);
        }

        unset($items[$index]);
    }

    private function getMenuItemIndex($id, $title, $items) {
        foreach ($items as $index => $item) {
            if ($item->id == $id && $item->title == $title) {
                return $index;
            }
        }

        return null;
    }
}

//When a JSession is created (for example, when creating the JApplication), it
//calls to "session_start()". It fails with
//"Cannot send session cookie - headers already sent by (output started at /usr/share/pear/PHPUnit/Util/Printer.php:174)"
//As the session is not relevant to test the module, a JSession stub with an
//empty "_start()" method (the one that called "session_start()" is used.
class JSessionStub extends JSession {
    protected function _start() {
    }
}
